Package: scRNAseqApp
Title: A single-cell RNAseq Shiny app-package
Version: 1.12.0
Authors@R: 
    c(person(given = "Jianhong",
             family = "Ou",
             role = c("aut", "cre"),
             email = "jou@morgridge.org",
             comment=c(ORCID="0000-0002-8652-2488")))
Description: The scRNAseqApp is a Shiny app package designed for
        interactive visualization of single-cell data. It is an
        enhanced version derived from the ShinyCell, repackaged to
        accommodate multiple datasets. The app enables users to
        visualize data containing various types of information
        simultaneously, facilitating comprehensive analysis.
        Additionally, it includes a user management system to regulate
        database accessibility for different users.
License: GPL-3
Encoding: UTF-8
Roxygen: list(markdown = TRUE)
RoxygenNote: 7.3.3
VignetteBuilder: knitr
biocViews: Visualization, SingleCell, RNASeq
Depends: R (>= 4.3.0)
Imports: bibtex, bslib, circlize, ComplexHeatmap, colourpicker,
        data.table, desc, DBI, DT, fs, GenomicRanges, GenomeInfoDb,
        ggdendro, ggforce, ggnewscale, ggplot2, ggrepel, ggridges,
        grDevices, grid, gridExtra, htmltools, IRanges, jsonlite,
        Matrix, magrittr, methods, patchwork, plotly, RColorBrewer,
        RefManageR, reshape2, rhdf5, Rsamtools, RSQLite, rtracklayer,
        S4Vectors, scales, scrypt, Seurat, SeuratObject, shiny,
        shinyhelper, shinymanager, slingshot, SingleCellExperiment,
        sortable, stats, tools, xfun, xml2, utils
Suggests: rmarkdown, knitr, testthat, BiocStyle, shinytest2
Enhances: celldex, future, SingleR, SummarizedExperiment, tricycle,
        terra
URL: https://github.com/jianhong/scRNAseqApp
BugReports: https://github.com/jianhong/scRNAseqApp/issues
Config/pak/sysreqs: cmake libfontconfig1-dev libfreetype6-dev
        libglpk-dev make libbz2-dev libicu-dev liblzma-dev libpng-dev
        libuv1-dev libxml2-dev libssl-dev perl python3 xz-utils
        zlib1g-dev
Repository: https://bioc-release.r-universe.dev
Date/Publication: 2026-04-28 12:59:37 UTC
RemoteUrl: https://github.com/bioc/scRNAseqApp
RemoteRef: RELEASE_3_23
RemoteSha: 96c8478c988d445bc6e16f5fb13f936abcc81fe0
NeedsCompilation: no
Packaged: 2026-04-30 06:28:39 UTC; root
Author: Jianhong Ou [aut, cre] (ORCID: <https://orcid.org/0000-0002-8652-2488>)
Maintainer: Jianhong Ou <jou@morgridge.org>
Built: R 4.6.0; ; 2026-04-30 07:16:47 UTC; unix
