# Generated by roxygen2: do not edit by hand

export(APPconf)
export(availableThemes)
export(createAppConfig)
export(createDataSet)
export(extractFragmentNameMapList)
export(scInit)
export(scRNAseqApp)
exportClasses(APPconf)
exportMethods("$")
exportMethods("$<-")
exportMethods("[")
exportMethods("[[")
exportMethods("[[<-")
exportMethods(as.character)
exportMethods(as.list)
exportMethods(lapply)
exportMethods(markers)
exportMethods(show)
exportMethods(unlist)
import(shiny)
importFrom(ComplexHeatmap,Heatmap)
importFrom(ComplexHeatmap,HeatmapAnnotation)
importFrom(ComplexHeatmap,Legend)
importFrom(ComplexHeatmap,draw)
importFrom(ComplexHeatmap,merge_dendrogram)
importFrom(ComplexHeatmap,pindex)
importFrom(ComplexHeatmap,rowAnnotation)
importFrom(DBI,dbAppendTable)
importFrom(DBI,dbClearResult)
importFrom(DBI,dbConnect)
importFrom(DBI,dbDisconnect)
importFrom(DBI,dbGetQuery)
importFrom(DBI,dbListFields)
importFrom(DBI,dbListTables)
importFrom(DBI,dbSendQuery)
importFrom(DBI,dbWriteTable)
importFrom(DBI,sqlInterpolate)
importFrom(DT,DTOutput)
importFrom(DT,JS)
importFrom(DT,datatable)
importFrom(DT,formatRound)
importFrom(DT,renderDT)
importFrom(GenomeInfoDb,`seqlevelsStyle<-`)
importFrom(GenomeInfoDb,keepSeqlevels)
importFrom(GenomeInfoDb,seqinfo)
importFrom(GenomeInfoDb,seqlevelsStyle)
importFrom(GenomeInfoDb,seqnames)
importFrom(GenomicRanges,GRanges)
importFrom(GenomicRanges,GRangesList)
importFrom(GenomicRanges,`end<-`)
importFrom(GenomicRanges,`start<-`)
importFrom(GenomicRanges,`strand<-`)
importFrom(GenomicRanges,coverage)
importFrom(GenomicRanges,end)
importFrom(GenomicRanges,gaps)
importFrom(GenomicRanges,makeGRangesFromDataFrame)
importFrom(GenomicRanges,reduce)
importFrom(GenomicRanges,seqnames)
importFrom(GenomicRanges,start)
importFrom(GenomicRanges,strand)
importFrom(GenomicRanges,width)
importFrom(IRanges,IRanges)
importFrom(IRanges,Views)
importFrom(IRanges,nearest)
importFrom(IRanges,ranges)
importFrom(IRanges,shift)
importFrom(IRanges,subsetByOverlaps)
importFrom(IRanges,tile)
importFrom(IRanges,viewMeans)
importFrom(Matrix,sparseMatrix)
importFrom(RColorBrewer,brewer.pal)
importFrom(RColorBrewer,brewer.pal.info)
importFrom(RSQLite,SQLite)
importFrom(RefManageR,GetBibEntryWithDOI)
importFrom(RefManageR,GetPubMedByID)
importFrom(RefManageR,PrintBibliography)
importFrom(Rsamtools,TabixFile)
importFrom(Rsamtools,scanTabix)
importFrom(Rsamtools,seqnamesTabix)
importFrom(S4Vectors,DataFrame)
importFrom(S4Vectors,SimpleList)
importFrom(S4Vectors,mcols)
importFrom(S4Vectors,split)
importFrom(Seurat,CellCycleScoring)
importFrom(Seurat,CreateSeuratObject)
importFrom(Seurat,FindAllMarkers)
importFrom(Seurat,FindVariableFeatures)
importFrom(Seurat,Read10X)
importFrom(Seurat,ScaleData)
importFrom(Seurat,as.SingleCellExperiment)
importFrom(SeuratObject,Assays)
importFrom(SeuratObject,CreateDimReducObject)
importFrom(SeuratObject,DefaultAssay)
importFrom(SeuratObject,DefaultBoundary)
importFrom(SeuratObject,DefaultFOV)
importFrom(SeuratObject,Embeddings)
importFrom(SeuratObject,GetAssayData)
importFrom(SeuratObject,GetTissueCoordinates)
importFrom(SeuratObject,Idents)
importFrom(SeuratObject,Images)
importFrom(SeuratObject,Misc)
importFrom(SeuratObject,Molecules)
importFrom(SeuratObject,Reductions)
importFrom(SeuratObject,VariableFeatures)
importFrom(SeuratObject,`DefaultAssay<-`)
importFrom(SeuratObject,`Idents<-`)
importFrom(SeuratObject,`Misc<-`)
importFrom(SingleCellExperiment,reducedDim)
importFrom(bibtex,read.bib)
importFrom(bslib,bs_current_theme)
importFrom(bslib,bs_get_variables)
importFrom(bslib,bs_theme)
importFrom(bslib,bs_themer)
importFrom(circlize,colorRamp2)
importFrom(colourpicker,colourInput)
importFrom(data.table,":=")
importFrom(data.table,.I)
importFrom(data.table,.N)
importFrom(data.table,.SD)
importFrom(data.table,as.data.table)
importFrom(data.table,data.table)
importFrom(data.table,dcast.data.table)
importFrom(data.table,fread)
importFrom(data.table,rbindlist)
importFrom(data.table,uniqueN)
importFrom(desc,desc)
importFrom(desc,desc_get_maintainer)
importFrom(fs,path_sanitize)
importFrom(ggdendro,dendro_data)
importFrom(ggforce,geom_arc_bar)
importFrom(ggforce,geom_bezier)
importFrom(ggforce,geom_circle)
importFrom(ggforce,geom_mark_hull)
importFrom(ggnewscale,new_scale_fill)
importFrom(ggplot2,.data)
importFrom(ggplot2,GeomPolygon)
importFrom(ggplot2,GeomViolin)
importFrom(ggplot2,aes)
importFrom(ggplot2,after_stat)
importFrom(ggplot2,coord_cartesian)
importFrom(ggplot2,coord_equal)
importFrom(ggplot2,coord_fixed)
importFrom(ggplot2,coord_flip)
importFrom(ggplot2,element_blank)
importFrom(ggplot2,element_line)
importFrom(ggplot2,element_rect)
importFrom(ggplot2,element_text)
importFrom(ggplot2,facet_grid)
importFrom(ggplot2,geom_bar)
importFrom(ggplot2,geom_boxplot)
importFrom(ggplot2,geom_col)
importFrom(ggplot2,geom_density_2d)
importFrom(ggplot2,geom_density_2d_filled)
importFrom(ggplot2,geom_hline)
importFrom(ggplot2,geom_jitter)
importFrom(ggplot2,geom_line)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,geom_polygon)
importFrom(ggplot2,geom_raster)
importFrom(ggplot2,geom_rect)
importFrom(ggplot2,geom_segment)
importFrom(ggplot2,geom_text)
importFrom(ggplot2,geom_tile)
importFrom(ggplot2,geom_violin)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,ggplot_build)
importFrom(ggplot2,ggplot_gtable)
importFrom(ggplot2,ggproto)
importFrom(ggplot2,ggsave)
importFrom(ggplot2,guide_colorbar)
importFrom(ggplot2,guide_legend)
importFrom(ggplot2,guides)
importFrom(ggplot2,labs)
importFrom(ggplot2,layer)
importFrom(ggplot2,layer_scales)
importFrom(ggplot2,position_dodge)
importFrom(ggplot2,scale_color_gradient2)
importFrom(ggplot2,scale_color_gradientn)
importFrom(ggplot2,scale_color_manual)
importFrom(ggplot2,scale_colour_manual)
importFrom(ggplot2,scale_fill_gradientn)
importFrom(ggplot2,scale_fill_manual)
importFrom(ggplot2,scale_x_continuous)
importFrom(ggplot2,scale_y_continuous)
importFrom(ggplot2,scale_y_discrete)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_classic)
importFrom(ggplot2,theme_minimal)
importFrom(ggplot2,unit)
importFrom(ggplot2,vars)
importFrom(ggplot2,xlab)
importFrom(ggplot2,xlim)
importFrom(ggplot2,ylab)
importFrom(ggplot2,ylim)
importFrom(ggrepel,geom_text_repel)
importFrom(ggridges,geom_density_ridges)
importFrom(ggridges,theme_ridges)
importFrom(grDevices,col2rgb)
importFrom(grDevices,colorRampPalette)
importFrom(grDevices,dev.off)
importFrom(grDevices,hcl.colors)
importFrom(grDevices,pdf)
importFrom(grDevices,pdfFonts)
importFrom(grDevices,rgb)
importFrom(grid,arrow)
importFrom(grid,convertWidth)
importFrom(grid,current.viewport)
importFrom(grid,gpar)
importFrom(grid,grid.circle)
importFrom(grid,grid.lines)
importFrom(grid,grid.xaxis)
importFrom(grid,grid.yaxis)
importFrom(grid,grobName)
importFrom(grid,grobTree)
importFrom(grid,seekViewport)
importFrom(grid,unit)
importFrom(grid,unit.c)
importFrom(gridExtra,arrangeGrob)
importFrom(gridExtra,grid.arrange)
importFrom(htmltools,htmlDependency)
importFrom(jsonlite,fromJSON)
importFrom(jsonlite,parse_json)
importFrom(magrittr,"%>%")
importFrom(methods,`slot<-`)
importFrom(methods,as)
importFrom(methods,getPackageName)
importFrom(methods,is)
importFrom(methods,new)
importFrom(methods,prototype)
importFrom(methods,representation)
importFrom(methods,setAs)
importFrom(methods,setClass)
importFrom(methods,setClassUnion)
importFrom(methods,setMethod)
importFrom(methods,show)
importFrom(methods,slot)
importFrom(methods,slotNames)
importFrom(patchwork,wrap_plots)
importFrom(plotly,event_data)
importFrom(plotly,event_register)
importFrom(plotly,ggplotly)
importFrom(plotly,layout)
importFrom(plotly,plot_ly)
importFrom(plotly,plotlyOutput)
importFrom(plotly,renderPlotly)
importFrom(plotly,toWebGL)
importFrom(reshape2,melt)
importFrom(rhdf5,H5Fclose)
importFrom(rhdf5,H5Fopen)
importFrom(rhdf5,H5Lexists)
importFrom(rhdf5,h5createDataset)
importFrom(rhdf5,h5createFile)
importFrom(rhdf5,h5createGroup)
importFrom(rhdf5,h5ls)
importFrom(rhdf5,h5read)
importFrom(rhdf5,h5write)
importFrom(rtracklayer,export)
importFrom(rtracklayer,import)
importFrom(scales,rescale)
importFrom(scales,zero_range)
importFrom(scrypt,hashPassword)
importFrom(shiny,Progress)
importFrom(shiny,downloadHandler)
importFrom(shiny,getDefaultReactiveDomain)
importFrom(shiny,markdown)
importFrom(shinyhelper,helper)
importFrom(shinyhelper,observe_helpers)
importFrom(shinymanager,auth_ui)
importFrom(shinymanager,check_credentials)
importFrom(shinymanager,create_db)
importFrom(shinymanager,fab_button)
importFrom(shinymanager,pwd_ui)
importFrom(shinymanager,read_db_decrypt)
importFrom(shinymanager,secure_app)
importFrom(shinymanager,secure_server)
importFrom(shinymanager,use_language)
importFrom(slingshot,SlingshotDataSet)
importFrom(slingshot,getLineages)
importFrom(slingshot,slingClusterLabels)
importFrom(slingshot,slingLineages)
importFrom(slingshot,slingMST)
importFrom(slingshot,slingParams)
importFrom(sortable,rank_list)
importFrom(stats,as.dendrogram)
importFrom(stats,as.formula)
importFrom(stats,chisq.test)
importFrom(stats,dist)
importFrom(stats,hclust)
importFrom(stats,prcomp)
importFrom(stats,quantile)
importFrom(stats,rnorm)
importFrom(stats,weighted.mean)
importFrom(stats,wilcox.test)
importFrom(tools,file_ext)
importFrom(utils,adist)
importFrom(utils,bibentry)
importFrom(utils,combn)
importFrom(utils,download.file)
importFrom(utils,getFromNamespace)
importFrom(utils,globalVariables)
importFrom(utils,head)
importFrom(utils,packageDescription)
importFrom(utils,packageVersion)
importFrom(utils,read.csv)
importFrom(utils,read.delim)
importFrom(utils,read.table)
importFrom(utils,write.csv)
importFrom(utils,write.table)
importFrom(xfun,base64_uri)
importFrom(xfun,is_abs_path)
importFrom(xml2,as_list)
importFrom(xml2,read_xml)
importMethodsFrom(Matrix,summary)
